
/*******************************************
 * Written by: Hui Fang (hfang@ece.udel.edu)
 * Last updated: 08/16/2009
 * For Lucene-2.9-dev
 * Retrieving documents with an axiomatic retrieval function ,i.e., F2-EXP 
 * References: H. Fang and C. Zhai. "An Exploration of Axiomatic Approaches to Information Retrieval", SIGIR'05. 
 ******************************************/

import java.io.*;
import java.util.*;

import org.apache.lucene.document.Document;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.Directory;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.Explanation;
 

public class RetAXEval {

  public static void main(String[] args) throws IOException{
      String usage = "java " + RetAXEval.class.getName() + " <index_dir> <query_file> <result_file> K(0.35) S(0.5)"; 
    if (args.length != 5) {
	System.err.println("Usage: " + usage);
	System.exit(1);
    }

    String indexDir = args[0]; 
    Directory dir = new FSDirectory(new File(indexDir), null); 

    float avdl=0.0f;     
    float paramK = Float.valueOf(args[3]).floatValue();
    float paramS = Float.valueOf(args[4]).floatValue();

    try {
      IndexSearcher searcher = new IndexSearcher(dir);

      PrintStream  outf = new PrintStream (                    
                            new BufferedOutputStream (
                                new FileOutputStream (args[2])));

      File fdl =new File(indexDir + "/LengthInfo.txt");
      FileInputStream infdl = new FileInputStream(fdl);
      InputStreamReader isrdl = new InputStreamReader(infdl, "US-ASCII");
      BufferedReader indl = new BufferedReader(isrdl);
      
      float sumT = 0.0f; 
      float numT = 0.0f; 
   
      while (true) {
	  String line = indl.readLine(); 
	  if (line == null) break; 
	  StringTokenizer st = new StringTokenizer(line, " ",false); 
	  String fName = st.nextToken(); 
	  String fsum = st.nextToken(); 
	  String fnum = st.nextToken(); 

	  if (fName.equals ("contents")) {
	      sumT += Float.valueOf(fsum).floatValue();  
	      numT += Float.valueOf(fnum).floatValue();  
	  }	  
      }

      avdl = sumT / numT; 


      File fin=new File(args[1]); 
      FileInputStream inf = new FileInputStream(fin);
      InputStreamReader isr = new InputStreamReader(inf, "US-ASCII");
      BufferedReader in = new BufferedReader(isr);

      try {
      	while (true) {
		String line = in.readLine();
		if (line == null) break;

                StringTokenizer st = new StringTokenizer(line, "\t", false);
		String qID=""; 
		String queryString=""; 
                try{
			qID = st.nextToken();
			queryString = st.nextToken();
	        } catch (NoSuchElementException e){
        	}
              
		BooleanQuery query = new BooleanQuery(true); 

		st = new StringTokenizer(queryString);
		while (st.hasMoreTokens()) {
                    AXTermQuery AXterm = new AXTermQuery(new Term("contents",st.nextToken()));
                    AXterm.setAvgDL(avdl);
                    AXterm.setParamK(paramK);
                    AXterm.setParamS(paramS);
                    query.add(AXterm, BooleanClause.Occur.SHOULD);
		}


		TopDocs hits = searcher.search(query,1000);
		for (int i = 0; i < hits.scoreDocs.length; i++ ) {
		  if (i<1000) {
		        ScoreDoc scoreDoc = hits.scoreDocs[i]; 
			Document doc = searcher.doc(scoreDoc.doc); 

			/*
			Explanation explanation = searcher.explain(query,scoreDoc.doc);
			System.out.println("-----------------");
			System.out.println("Docid:"+doc.get("DocID"));
			System.out.println(explanation.toString());
			*/

	    		String path = doc.get("DocID");
	    		if (path != null) {
				outf.println(qID + " " + path + " " +scoreDoc.score);
	    		} 
		  }	
	  	}
	}
      } catch (IOException e) {
	      System.out.println(" caught a " + e.getClass() +
			 "\n with message: " + e.getMessage());
      }
	
      inf.close();
      outf.close();
      searcher.close();

    } catch (Exception e) {
      System.out.println(" caught a " + e.getClass() +
			 "\n with message: " + e.getMessage());
    }
  }
}
